# SPDX-FileCopyrightText: 2025 Lubin Bailly <lubin.bailly@dgnum.eu>
#
# SPDX-License-Identifier: EUPL-1.2

{ lib, config, ... }:
let
  inherit (lib)
    filter
    genList
    mapAttrs
    mkEnableOption
    mkOption
    mkIf
    optional
    optionals
    ;
  inherit (lib.types)
    ints
    ;

  cfg = config.dgn-hackdays;

  idxInfo = idx: rec {
    prefix24 = idx * 4;
    next-hop = "192.168.0.${toString (1 + idx)}";
    destination = "192.168.${toString prefix24}.0/22";
    internal = "192.168.${toString prefix24}.1";
  };

  inherit (idxInfo cfg.index)
    prefix24
    next-hop
    destination
    internal
    ;
  dhcpRange = {
    low = "192.168.${toString prefix24}.2";
    high = "192.168.${toString (prefix24 + 3)}.250";
  };
in
{
  options.dgn-hackdays = {
    enable = mkEnableOption "hackdays configuration";
    portConfig = mkEnableOption "default port configuration for hackdays switches" // {
      default = true;
    };
    index = mkOption {
      type = ints.between 1 30;
      description = ''
        Identifier of this switch.
      '';
    };
  };
  config = mkIf cfg.enable {
    dgn-hardware.model = "EX4100-F-48P";
    vlans = {
      "hackdays" = {
        id = 998;
        l3-interface = "irb.1";
      };
      "uplink" = {
        id = 999;
        l3-interface = "irb.0";
      };
    };
    dgn-profiles = {
      "client" = {
        interfaces = optionals cfg.portConfig (genList (i: "ge-0/0/${toString i}") 47);
        configuration = {
          poe = true;
          ethernet-switching = {
            interface-mode = "access";
            vlans = [ "hackdays" ];
          };
        };
      };
      "uplink" = {
        interfaces = optional cfg.portConfig "ge-0/0/47";
        configuration.ethernet-switching = {
          interface-mode = "access";
          vlans = [ "uplink" ];
          rstp = false;
        };
      };
    };

    # dgn-interfaces is oversimplified for irb
    interfaces."irb" = {
      enable = true;
      unit =
        mapAttrs
          (_: addr: {
            family.inet = {
              enable = true;
              addresses = [ addr ];
            };
          })
          {
            "0" = "${next-hop}/22";
            "1" = "${internal}/22";
          };
    };

    system.services.dhcp-local-server.group."hackdays".interfaces = [ "irb.1" ];
    access.address-assignment.pool."hackdays".family.inet = {
      network = destination;
      ranges = [ dhcpRange ];
      dhcp-attributes = {
        maximum-lease-time = 7200;
        router = [ internal ];
        name-server = [
          "1.1.1.1"
          "1.0.0.1"
          "8.8.8.8"
          "8.8.4.4"
        ];
      };
    };
    routing-options.static.route =
      [
        {
          destination = "0.0.0.0/0";
          next-hop = "192.168.0.1";
        }
      ]
      ++ map (idx: { inherit (idxInfo idx) destination next-hop; }) (
        filter (i: i != cfg.index) (genList (i: i + 1) 30)
      );
  };
}
